@@ -13,9 +13,9 @@
 #include <cuda.h>
 #include <torch/types.h>
 
-#include "src/torchcodec/_core/NVCUVIDRuntimeLoader.h"
-#include "src/torchcodec/_core/nvcuvid_include/cuviddec.h"
-#include "src/torchcodec/_core/nvcuvid_include/nvcuvid.h"
+#include "NVCUVIDRuntimeLoader.h"
+#include "nvcuvid_include/cuviddec.h"
+#include "nvcuvid_include/nvcuvid.h"
 
 namespace facebook::torchcodec {
 
