@@ -19,6 +19,7 @@ from .ops import (
     _test_frame_pts_equality,
     add_audio_stream,
     add_video_stream,
+    core_library_path,
     create_from_bytes,
     create_from_file,
     create_from_file_like,
@@ -29,6 +30,7 @@ from .ops import (
     encode_video_to_file,
     encode_video_to_file_like,
     encode_video_to_tensor,
+    ffmpeg_major_version,
     get_ffmpeg_library_versions,
     get_frame_at_index,
     get_frame_at_pts,
